#!/bin/bash
#Description: DevKit constants
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2023. All rights reserved.
#History: 2023-2-9 created
current_dir=$(cd $(dirname $0); pwd)
source "${current_dir}/const.sh"
source "${current_dir}/utils.sh"

function init_variable() {
	if [ ! -f "${INSTALL_PATH}/tools/log.sh" ]; then
		echo "There is no log file(${INSTALL_PATH}/tools/log.sh), please check!"
		exit 1
	fi
	source ${INSTALL_PATH}/tools/log.sh
	INSTALL_INI_PATH="${INSTALL_PATH}/config/install.ini"
	if [[ ! -f "${INSTALL_INI_PATH}" ]]; then
		echo_and_log "There is no configure file(${INSTALL_INI_PATH}), please check!" "${ERROR}"
		exit "${ERROR}"
	fi
	USER_PATH="$(grep -w USER_PATH "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')"
	USER_NAME="$(grep -w USER_NAME "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')"
	gunicorn_ip=$(grep -w GUNICORN_IP "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')
	gunicorn_port=$(grep -w PLUGIN_GUNICORN_PORT "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')
	os_type=$(grep -w OS_TYPE "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')
	os_version=$(grep -w OS_VERSION "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')
	plugin_name="porting"
	INSTALL_LOG_PATH="${INSTALL_PATH}/logs/install_plugins.log"
	INSTALL_PLUGIN_PATH="${INSTALL_PATH}/devkitplugins"
	# 插件压缩包存档路径
	PYTHON_PATH="${INSTALL_PATH}/tools/python_venv/bin/python3"

	DEPENDENCY_DICTIONARY_PATH="${INSTALL_PATH}/devkitplugins/porting/config/restore_dependency_package"
	dependency_dictionary_name=$(ls ${DEPENDENCY_DICTIONARY_PATH} | grep "Dependency-Dictionary")
	MIGRATION_PATH="${INSTALL_PATH}/devkitplugins/porting/config/restore_dependency_package"
	migration_name=$(ls ${MIGRATION_PATH} | grep "Migration-Package")
}

function change_file_permission() {
    local file="$1"
    local num="$2"
    if [ -f "${file}" ]; then
        chmod "${num}" "${file}"
    fi
}

function install_dependency_dictionary_status() {
	local sqlite3_bin_path=$(which sqlite3)
	local sqlite3_file_path="${INSTALL_PATH}/devkitplugins/sqlite/db.sqlite3"
	${sqlite3_bin_path} "${sqlite3_file_path}" "insert into dependency_dictionary_dependencydictionary (file_name, file_source) values ('${dependency_dictionary_name}', 0);"
}

function change_file_path_permission() {
	# 修改文件的权限
	all_asm_path=${INSTALL_PATH}/devkitplugins/${plugin_name}/all_asm
	inline_asm_path=${INSTALL_PATH}/devkitplugins/${plugin_name}/inline_asm
	if [ -d ${all_asm_path} ] && [ -d ${inline_asm_path} ]; then
		chmod -R 500 ${all_asm_path}/bin
		chmod 600 ${all_asm_path}/config/AllAsmLogCfg.json
		chmod 400 ${all_asm_path}/config/dependency.conf
		chmod -R 400 ${all_asm_path}/lib
		chmod 700 ${all_asm_path}/bin ${all_asm_path}/lib ${all_asm_path}/lib/semantics

		chmod -R 500 ${inline_asm_path}/clang-tidy/bin/* ${inline_asm_path}/clang-tidy/lib/* ${inline_asm_path}/remill/bin/* ${inline_asm_path}/remill/lib/*
		chmod -R 400 ${inline_asm_path}/clang-tidy/include/* ${inline_asm_path}/remill/lib/semantics/*
		chmod -R 600 ${inline_asm_path}/config/*
		chmod 700 $(find ${inline_asm_path}/clang-tidy/include/ -type d)
		chmod 700 ${inline_asm_path}/remill/lib/semantics/
	fi

	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/decode_gcm.so" 400
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/log.conf" 600
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/log/${plugin_name}.json" 600
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/porting_rules.json" 600
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/ignore_rules.json" 600

	change_file_permission "${DEPENDENCY_DICTIONARY_PATH}/${dependency_dictionary_name}" 600
	change_file_permission "${MIGRATION_PATH}/${migration_name}" 600
	chmod 700 $(find ${INSTALL_PATH}/devkitplugins/porting/resource/migration -type f)
	chmod 600 $(find ${INSTALL_PATH}/devkitplugins/porting/resource/migration -regex ".*\.cipher")
}

function insert_to_sudoers() {
	# 暂时只支持Kunpeng, CentOS7.6和openEuler20.03sp1上进行解决方案移植
	if [[ $(uname -m) == "aarch64" ]]; then
		openeuler_version=$(cat /etc/os-release | grep VERSION= | awk -F '"' '{print$2}')
		if [[ "$os_type" == "CentOS" && ${os_version} == "7.6" ]] ||
			[[ -f /etc/openEuler-release && ${openeuler_version} == "20.03 (LTS-SP1)" ]]; then
			add_devkit_plugin_sudo_list
			if [ "0" != "$?" ]; then
				exit ${ERROR}
			fi
		fi
	fi
}

function add_devkit_sudo() {
	local install_log="$3"
	if [ ! -f "${sudo_file}" ]; then
		touch $sudo_file
	fi
	chmod 440 $sudo_file
	command="$1"
	command_path="$2"
	if grep "${USER_NAME}    ALL=(root)      NOPASSWD:${command_path}" "$sudo_file"; then
		echo_and_log "${command} is already in the ${USER_NAME} sudoer list." "${DOING}" ${install_log}
	else
		# 空文件sed 追加不进去 sed 根据行号追加内容
		echo "${USER_NAME}    ALL=(root)      NOPASSWD:""${command_path}" >>"$sudo_file"
	fi
}

function modify_spec_default_value() {
	# 修改专项软件迁移中spec以及sh 文件中的写死的/home/devkit
	migration_path="${INSTALL_PATH}/devkitplugins/porting/resource/migration"
	if [[ "$(grep -r "/home/devkit" ${migration_path})" ]]; then
	    grep -r "/home/devkit" ${migration_path} | awk -F ":" '{print $1}' | xargs sed -i "s#/home/devkit#${USER_PATH}#"g
	fi
}

function add_devkit_plugin_sudo_list() {
	centos_parameter_list=(
		"/usr/bin/yum -y install curl tar coreutils redhat-lsb* krb5-devel perl-interpreter pcre pcre-devel sed rpm-build gcc gcc-c++ make libtool zlib zlib-devel perl*"
		"/usr/bin/yum -y install zlib-devel texinfo gstreamer-plugins-base-devel libXext-devel libGLU-devel libXt-devel libXrender-devel libXinerama-devel libpng-devel libXrandr-devel libXi-devel libXft-devel libjpeg-turbo-devel libXcursor-devel readline-devel ncurses-devel"
		"/usr/bin/yum -y groupinstall Development Tools"
		"/usr/bin/rpm -ivh ${USER_PATH}/rpmbuild/RPMS/aarch64/openssl-libs-1.1.1a-3.el7.aarch64.rpm ${USER_PATH}/rpmbuild/RPMS/aarch64/openssl-devel-1.1.1a-3.el7.aarch64.rpm ${USER_PATH}/rpmbuild/RPMS/aarch64/openssl-1.1.1a-3.el7.aarch64.rpm --force --nodeps"
		"/usr/bin/tee -a /etc/ld.so.conf"
		"/usr/bin/yum -y install csh glibc-static time"
		"/usr/bin/yum -y install zlib* lapack* blas* python3.aarch64 python3-devel.aarch64 cmake3* boost_python* boost boost-devel numpy* python36-numpy flex bison tk wget"
		"/usr/bin/yum -y install perl-URI"
	)
	openEuler_parameter_list=(
		"/usr/bin/yum -y install curl patch openssl-devel zlib-devel automake libtool make libstdc++-static glibc-static git ant gcc-c++ gcc-gfortran libgcc libXt-devel readline-devel"
		"/usr/bin/yum -y install curl patch openssl-devel zlib-devel automake libtool make libstdc++-static glibc-static git gcc-c++ gcc-gfortran libgcc protobuf protobuf-devel"
		"/usr/bin/yum -y install curl patch openssl-devel zlib-devel automake libtool make libstdc++-static glibc-static git gcc-c++ gcc-gfortran libgcc"
		"/usr/bin/yum -y install curl tar coreutils openeuler-lsb krb5-devel perl-interpreter pcre pcre-devel sed rpm-build gcc gcc-c++ make libtool zlib zlib-devel perl*"
		"/usr/bin/yum -y install bison ncurses ncurses-devel libaio-devel openssl openssl-devel gmp gmp-devel mpfr mpfr-devel libmpc libmpc-devel wget tar gcc gcc-c++ git rpcgen cmake libtirpc-devel openldap-devel openldap net-tools"
		"/usr/bin/yum -y install gcc gcc-c++ make openeuler-lsb libtool zlib zlib-devel pcre pcre-devel perl-devel perl-ExtUtils-Embed openssl openssl-devel net-tools* wget"
		"/usr/bin/yum -y install zlib-devel texinfo gstreamer-plugins-base-devel libXext-devel libGLU-devel libXt-devel libXrender-devel libXinerama-devel libpng-devel libXrandr-devel libXi-devel libXft-devel libjpeg-turbo-devel libXcursor-devel readline-devel ncurses-devel"
		"/usr/bin/yum -y groupinstall Development Tools"
		"/usr/bin/yum -y install perl-URI zlib zlib-devel boost-* perl-*"
		"/usr/bin/yum -y install csh time patch libatomic perl-Data-Dumper autoconf automake libtool numactl binutils systemd-devel valgrind flex wget"
		"/usr/bin/yum -y install time patch libatomic autoconf automake libtool numactl binutils systemd-devel valgrind flex wget cmake"
		"/usr/bin/yum -y install libicu libicu-devel zlib zlib-devel bzip2 bzip2-devel"
		"/usr/bin/yum -y install zlib* lapack* blas* python3.aarch64 python3-devel.aarch64 cmake3 boost-python* boost boost-devel numpy* python-numpy flex bison tcl tk"
		"/usr/bin/yum -y install zlib-devel texinfo gstreamer-plugins-base-devel libXext-devel libGLU-devel libXt-devel libXrender-devel libXinerama-devel libpng-devel libXrandr-devel libXi-devel libXft-devel libjpeg-turbo-devel libXcursor-devel readline-devel ncurses-devel mpfr-devel gmp-devel libmpc*"
	)
	if [[ -f /etc/openEuler-release ]]; then
		local parameter_list_len=${#openEuler_parameter_list[@]}
	else
		local parameter_list_len=${#centos_parameter_list[@]}
	fi
	sudo_file="/etc/sudoers.d/devkit_porting_sudoers"
	for ((i = 0; i < ${parameter_list_len}; i++)); do
		if [[ -f /etc/openEuler-release ]]; then
			local parameter=${openEuler_parameter_list[i]}
		else
			local parameter=${centos_parameter_list[i]}
		fi
		local cmd_name=$(echo ${parameter} | cut -d " " -f 1)
		add_devkit_sudo "${cmd_name}" "${parameter}" "${INSTALL_LOG_PATH}"
		if [ "0" != "$?" ]; then
			exit ${ERROR}
		fi
	done
}

# 参数2个，为插件名,和是否为接口调用 手动安装添加验签
function main() {
	# 用户通过接口调用安装
	INSTALL_PATH="$1" # /opt/DevKit
	plugin_flag="$2"  # web
	init_variable
	if [[ "${plugin_flag}" == "web" ]]; then
		bash "${current_dir}/env_check.sh" "${INSTALL_PATH}" "${plugin_flag}"
		local check_res="$?"
		export LD_LIBRARY_PATH=${DEVKIT_HOME}/libsqlite3:${DEVKIT_HOME}/libssl:${DEVKIT_HOME}/lib
        ${PYTHON_PATH} "${current_dir}/make_env_check_result.pyc"
        unset LD_LIBRARY_PATH
        local temp_env_json="${current_dir}/porting_env_res.json"
        local final_env_json="${INSTALL_PATH}/devkitplugins/check_env_result/porting_env_res.json"
        if [[ -f "${temp_env_json}" ]]; then
            mv "${temp_env_json}" "${final_env_json}"
            chmod 600 "${final_env_json}"
            chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${final_env_json}"
        fi
        if [[ "${check_res}" != "${SUCCESS}" ]]; then
            echo_and_log "Check install Dependent Packages Failed" "${ERROR}" "${INSTALL_LOG_PATH}"
            exit "${ERROR}"
        else
            echo_and_log "Check install Dependent Packages Success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
        fi
    else
    	# 磁盘空间大小检测
        echo_and_log "Check disk space" "${DOING}" "${INSTALL_LOG_PATH}"
        disk_space_check "${INSTALL_PATH}" "${INSTALL_SPACE_NEED}"
        if [ "$?" -ne "${SUCCESS}" ]; then
            echo_and_log "Check install path disk space Failed" "${ERROR}" "${INSTALL_LOG_PATH}"
            exit "${ERROR}"
        fi
        echo_and_log "Check install path disk space Success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
	fi

	insert_to_sudoers
	change_file_path_permission
	install_dependency_dictionary_status
	modify_spec_default_value
    # 删除依赖检查配置文件
	if [[ -e "${INSTALL_PATH}/devkitplugins/${plugin_name}/deploy/porting_env_res.ini" ]]; then
	    rm -rf "${INSTALL_PATH}/devkitplugins/${plugin_name}/deploy/porting_env_res.ini"
	fi
}

main $@
