#!/bin/bash

set -e

start_time=$(date +%s)
set_color(){
	echo -e "\n\033[$1######################################\n$2 \
		         \n#############################################\033[0m\n"
}

set_color 32m \
	"【部署须知】
1、本自动化安装脚本仅适用于Kunpeng平台;
2、本脚本部署 CP2K 版本
3、本脚本部署安装使用 gnu9.3 和 openmpi4.0.3
4、中途退出，请按Ctrl+C结束
"


#环境变量设置
#module use /Oceanfile/RR3-compiler-env/module_file/aarch64
#module add gnu-9.3.0 omp4.0.3-gcc9.3.0
export CC=`which mpicc`
export CXX=`which mpic++`
export FC=`which mpifort`

#设置安装包路径
export INSTALL_DIR=${PWD}
export PACKAGE_DIR=${INSTALL_DIR}
export CMAKE_DIR=${INSTALL_DIR}/CMAKE
export GMP_DIR=${INSTALL_DIR}/GMP
export BOOST_DIR=${INSTALL_DIR}/BOOST
export LIBINT_DIR=${INSTALL_DIR}/LIBINT
export FFTW_DIR=${INSTALL_DIR}/FFTW
export OPENBLAS_DIR=${INSTALL_DIR}/OPENBLAS
export SCALAPACK_DIR=${INSTALL_DIR}/SCALAPACK
export ELPA_DIR=${INSTALL_DIR}/ELPA
export SPGLIB_DIR=${INSTALL_DIR}/SPGLIB
export LIBXC_DIR=${INSTALL_DIR}/LIBXC
export GSL_DIR=${INSTALL_DIR}/GSL
export PLUMED_DIR=${INSTALL_DIR}/PLUMED
export EXTRA_DIR=${INSTALL_DIR}/EXTRA
export MATHLIB_DIR=${EXTRA_DIR}/mathlib
export CP2K_DIR=${INSTALL_DIR}/CP2K
export ENV_DIR=${INSTALL_DIR}/ENV
export LOGS_DIR=${INSTALL_DIR}/LOGS
env_check(){
# check Arch
        if [ x$(arch) = xaarch64 ];then
                set_color 32m "Arch check ok"
        else
                set_color 31m "该脚本仅支持Kunpeng平台，请检查后再执行。。。"
        fi

# check install dir
        if [ -d ${INSTALL_DIR} ];then
                echo "本次软件安装目录为：${INSTALL_DIR}"
        else
                echo"设置的软件目录不存在，请检查"
                exit 1
        fi
                                               
DIRS=(
${CMAKE_DIR}
${GMP_DIR}
${BOOST_DIR}
${LIBINT_DIR}
${FFTW_DIR}
${OPENBLAS_DIR}
${SCALAPACK_DIR}
${ELPA_DIR}
${SPGLIB_DIR}
${LIBXC_DIR}
${GSL_DIR}
${PLUMED_DIR}
${EXTRA_DIR}
${MATHLIB_DIR}
${ENV_DIR}
${LOGS_DIR})
for i in ${DIRS[@]}
do
 if [ -d $i ];then
   set_color 31m "This directory($i) already exists"
   read -p "Please choice backup or delete $i:(b|d)" choice
 case $choice in
 b)
   set_color 32m "You have selected file backup"
   mv $i $i-bak && mkdir -p $i
   set_color 32m "Backup file done"
 ;;
 d)
    set_color 31m "You have selected file delete"
    rm -rf $i && mkdir -p $i
    set_color 31m "Delete file done"
  ;;
 *)
    set_color 31m "Please input 'b' or 'd'"
    exit 1
  ;;
  esac
  else
    mkdir -p $i
    set_color 32m "$i创建成功"
  fi
done
}

check_install_result(){
                  app=$1
                  ls ${INSTALL_DIR}/${app} | wc -l
                if [ $? -eq 1 ];then
                            set_color 32m "$app 检查正常，请继续..."
                else
                            set_color 31m "$app 安装异常，安装中止..."
                            exit 1
                fi
}

#########################################################################
#           check_packages(安装包的版本必须一致并在同一路径下)          #
#########################################################################
CMAKE_package=cmake-3.16.4.tar.gz
GMP_package=gmp-6.2.0.tar.xz
BOOST_package=boost_1_72_0.tar.gz
LIBINT_package=libint-2.6.0.tar.gz
FFTW_package=fftw-3.3.8.tar.gz
OPENBLAS_package=OpenBLAS-0.3.7.tar.gz
SCALAPACK_package=scalapack-2.1.0.tgz
ELPA_package=elpa-2019.05.001.tar.gz
SPGLIB_package=spglib-1.11.2.1.tar.gz
LIBXC_package=libxc-4.3.4.tar.gz
GSL_package=gsl-2.6.tar.gz
PLUMED_package=plumed-2.5.2.tgz
LIBINT_CP2K_LMAX4_package=libint_cp2k_lmax4.tgz

########################################################################
#	                       use_package																	 #
########################################################################
cmake_dir_name=cmake-3.16.4
gmp_dir_name=gmp-6.2.0
boost_dir_name=boost_1_72_0
libint_dir_name=libint-2.6.0
fftw_dir_name=fftw-3.3.8
openblas_dir_name=OpenBLAS-0.3.7
scalapack_dir_name=scalapack-2.1.0
elpa_dir_name=elpa-2019.05.001
spglib_dir_name=spglib-1.11.2.1
libxc_dir_name=libxc-4.3.4
gsl_dir_name=gsl-2.6
plumed_dir_name=plumed-2.5.2
libint_cp2k_lmax4_dir_name=libint_cp2k_lmax4
#########################################################################
#                       CMAKE_package安装                               #
#########################################################################
install_cmake(){
tar -zxf ${PACKAGE_DIR}/${CMAKE_package} -C ${CMAKE_DIR}
cd ${CMAKE_DIR}/${cmake_dir_name}
./configure --prefix=${CMAKE_DIR}
make -j
make install
###设置CMAKE环境变量###
export PATH=${CMAKE_DIR}/bin:$PATH
}

#########################################################################
#                         GMP_package安装                               #
#########################################################################
install_gmp(){
tar -xf ${PACKAGE_DIR}/${GMP_package} -C ${GMP_DIR}
cd ${GMP_DIR}/${gmp_dir_name}
./configure --prefix=${GMP_DIR}
make -j
make install
}

########################################################################
#                        BOOST_package安装                             #
########################################################################
install_boost(){
tar -zxf ${PACKAGE_DIR}/${BOOST_package} -C ${BOOST_DIR}
cd ${BOOST_DIR}/${boost_dir_name}
./bootstrap.sh
./b2 install --prefix=${BOOST_DIR}
}

########################################################################
#                        LIBINT_package安装                            #
########################################################################
install_libint(){
tar -xf ${PACKAGE_DIR}/${LIBINT_package} -C ${LIBINT_DIR}
cd ${LIBINT_DIR}/${libint_dir_name}
./autogen.sh
mkdir build
cd build
###设置LIBINT所需环境变量###
export LDFLAGS="-L${GMP_DIR}/lib -L${BOOST_DIR}/lib"
export CPPFLAGS="-I${BOOST_DIR}/include  -I${GMP_DIR}/include"
../configure CXX=mpicxx --enable-eri=1 --enable-eri2=1 \
--enable-eri3=1 --with-max-am=4 --with-eri-max-am=4,3 \
--with-eri2-max-am=6,5 --with-eri3-max-am=6,5 --with-opt-am=3 --enable-generic-code --disable-unrolling --with-libint-exportdir=libint_cp2k_lmax4

make export

tar -xf ${LIBINT_CP2K_LMAX4_package}
cd ${libint_cp2k_lmax4_dir_name}
mkdir -p ${EXTRA_DIR}/libint2
./configure --prefix=${EXTRA_DIR}/libint2 CC=mpicc CXX=mpicxx FC=mpifort --enable-fortran --enable-shared
make -j32
make install
}

#######################################################################
#                       FFTW_package安装                              #
#######################################################################
install_fftw(){
tar -xf ${PACKAGE_DIR}/${FFTW_package} -C ${FFTW_DIR}
cd ${FFTW_DIR}/${fftw_dir_name}
mkdir -p ${EXTRA_DIR}/fftw3
./configure CC=gcc F77=gfortran --enable-shared --enable-threads --enable-openmp --enable-mpi MPICC=mpicc --prefix=${EXTRA_DIR}/fftw3
make -j
make install
}

#######################################################################
#                       OPENBLAS_package安装                            #
#######################################################################
install_openblas()
{
  export CC=$(which gcc)
  export CXX=$(which g++)
  export FC=$(which gfortran)
  tar -zxvf $PACKAGE_DIR/$OPENBLAS_package -C $OPENBLAS_DIR
  cd $OPENBLAS_DIR/${openblas_dir_name}
  make
  make PREFIX=$OPENBLAS_DIR install
  export LD_LIBRARY_PATH=$OPENBLAS_DIR/lib:$LD_LIBRARY_PATH
  cp $OPENBLAS_DIR/lib/*.a ${MATHLIB_DIR}
  export CC=`which mpicc`
  export CXX=`which mpic++`
  export FC=`which mpifort`
}

#######################################################################
#                       SCALAPACK_package安装                         #
#######################################################################
install_scalapack(){
tar -xf ${PACKAGE_DIR}/${SCALAPACK_package} -C ${SCALAPACK_DIR}
cd ${SCALAPACK_DIR}/${scalapack_dir_name}
cp SLmake.inc.example SLmake.inc
sed  -i "58cBLASLIB    = $OPENBLAS_DIR/lib/libopenblas.a" ${SCALAPACK_DIR}/${scalapack_dir_name}/SLmake.inc
sed  -i "59cLAPACKLIB  = $OPENBLAS_DIR/lib/libopenblas.a" ${SCALAPACK_DIR}/${scalapack_dir_name}/SLmake.inc

make
cp  *a ${MATHLIB_DIR}
}

#######################################################################
#                       ELPA_package安装                              #
#######################################################################
install_elpa(){
tar -xf ${PACKAGE_DIR}/${ELPA_package} -C ${ELPA_DIR}
cd ${ELPA_DIR}/${elpa_dir_name}
mkdir -p ${EXTRA_DIR}/elpa
./configure --prefix=${EXTRA_DIR}/elpa --enable-openmp --enable-shared=no LIBS="${MATHLIB_DIR}/libscalapack.a ${MATHLIB_DIR}/libopenblas.a ${MATHLIB_DIR}/libopenblas.a" --disable-sse --disable-sse-assembly --disable-avx --disable-avx2
make
make install
}

#######################################################################
#                       SPGLIB_package安装                            #
#######################################################################
install_spglib(){
tar -xf ${PACKAGE_DIR}/${SPGLIB_package} -C ${SPGLIB_DIR}
cd ${SPGLIB_DIR}/${spglib_dir_name}
mkdir -p ${SPGLIB_DIR}/${spglib_dir_name}/build
cd ${SPGLIB_DIR}/${spglib_dir_name}/build
mkdir -p ${EXTRA_DIR}/spglib112
cmake .. -DCMAKE_INSTALL_PREFIX="${EXTRA_DIR}/spglib112"
make -j
make install
}

#######################################################################
#                       LIBXC_package安装                             #
#######################################################################
install_libxc(){
tar -xf ${PACKAGE_DIR}/${LIBXC_package} -C ${LIBXC_DIR}
cd ${LIBXC_DIR}/${libxc_dir_name}
mkdir -p ${EXTRA_DIR}/libxc434
./configure FC=gfortran CC=gcc --prefix=${EXTRA_DIR}/libxc434
make
make install
}

#######################################################################
#                        GSL_package安装                              #
#######################################################################
install_gsl(){
tar -xf ${PACKAGE_DIR}/${GSL_package} -C ${GSL_DIR}
cd ${GSL_DIR}/${gsl_dir_name}
mkdir -p ${EXTRA_DIR}/gsl
./configure --prefix=${EXTRA_DIR}/gsl
make
make install

###设置所需GSL环境变量###
export LIBRARY_PATH=${EXTRA_DIR}/gsl/lib:\$LIBRARY_PATH
export LD_LIBRARY_PATH=${EXTRA_DIR}/gsl/lib:\$LD_LIBRARY_PATH
}

#######################################################################
#                       PLUMED_package安装                            #
#######################################################################
install_plumed(){
tar -xf ${PACKAGE_DIR}/${PLUMED_package} -C ${PLUMED_DIR}
cd ${PLUMED_DIR}/${plumed_dir_name}
mkdir -p ${EXTRA_DIR}/plumed252
./configure CXX=mpicxx CC=mpicc FC=mpifort --prefix=${EXTRA_DIR}/plumed252 --enable-external-blas --enable-gsl --enable-external-lapack LDFLAGS=-L${EXTRA_DIR}/mathlib LIBS="-lopenblas"
make
make install
}

install_all(){
install_cmake
install_gmp
install_boost
install_libint
install_fftw
install_openblas
install_scalapack
install_elpa
install_spglib
install_libxc
install_gsl
install_plumed
}


time_tag=$(echo $(date +%F-%H%M%S)_$(hostname))
env_check
install_all 2>&1 | tee -a  $LOGS_DIR/install_CP2K_dependencies_aarch64_gcc_${time_tag}.log
end_time=$(date +%s)
spend_time=$( expr ${end_time} - ${start_time} )
set_color 33m "CP2K依赖库安装成功，本次安装耗时：${spend_time}s"
