#!/bin/bash
set -e

WORK_HOME=`pwd`
INS_HOME_PATH="${WORK_HOME}/application"

max_jobs=$(nproc)
if [[ ${max_jobs} -gt 32 ]]; then
    max_jobs=32
fi
export BISHENG=$(cd $(dirname $(which clang))/../;pwd)
export PATH=${BISHENG}/bin:$PATH
export LD_LIBRARY_PATH=${BISHENG}/lib:$LD_LIBRARY_PATH
export INCLUDE=${BISHENG}/include:${WORK_HOME}/FFTW/include:$INCLUDE
export LIBEXEC=${BISHENG}/libexec:$LIBEXEC
export PATH=${WORK_HOME}/FFTW/bin:${WORK_HOME}/OpenBLAS/bin:${PATH}
export LD_LIBRARY_PATH=${WORK_HOME}/FFTW/lib:${WORK_HOME}/OpenBLAS/bin:${LD_LIBRARY_PATH}
#设置环境
FFTW_LIB=${WORK_HOME}/FFTW/lib
FFTW_INC=${WORK_HOME}/FFTW/include
BLAS_LIB=${WORK_HOME}/OpenBLAS/lib


function gromacs_install()
{
    cd ${WORK_HOME}/gromacs-2019.3
    mkdir build && cd build

    FLAGS="-mcpu=tsv110"; CFLAGS=$FLAGS CXXFLAGS=$FLAGS LDFLAGS="-lflang" CC=mpicc CXX=mpicxx \
    cmake \
    -DCMAKE_INSTALL_PREFIX=${INS_HOME_PATH}/GROMACS/ \
    -DBUILD_SHARED_LIBS=on \
    -DBUILD_TESTING=on \
    -DREGRESSIONTEST_DOWNLOAD=off \
    -DGMX_BUILD_OWN_FFTW=off \
    -DGMX_SIMD=ARM_NEON_ASIMD \
    -DGMX_DOUBLE=off \
    -DGMX_EXTERNAL_BLAS=on \
    -DGMX_EXTERNAL_LAPACK=on \
    -DGMX_FFT_LIBRARY=fftw3 \
    -DGMX_BLAS_USER=${BLAS_LIB}/libopenblas.a \
    -DGMX_LAPACK_USER=${BLAS_LIB}/libopenblas.a \
    -DFFTWF_LIBRARY=${FFTW_LIB}/libfftw3f.so \
    -DFFTWF_INCLUDE_DIR=${FFTW_INC} \
    -DGMX_GPU=off \
    -DGMX_MPI=on \
    -DGMX_OPENMP=on \
    -DGMX_X11=off \
    -DCMAKE_C_COMPILER_WORKS=1 \
    -DCMAKE_CXX_COMPILER_WORKS=1 \
    ../ 2>&1 | tee gromacs_cmake.log
    make -j ${max_jobs} V=1
    make -j ${max_jobs} install  2>&1 | tee gromacs_install.log
    cp gromacs_cmake.log gromacs_install.log ${INS_HOME_PATH}
}

gromacs_install
