#!/bin/bash
set -e

WORK_HOME=`pwd`
INS_HOME_PATH="${WORK_HOME}/application"
mkdir -p $INS_HOME_PATH

max_jobs=$(nproc)
if [[ ${max_jobs} -gt 32 ]]; then
    max_jobs=32
fi

#设置环境
export FFTW_DIR=${WORK_HOME}/FFTW
export MATH_LIB=${WORK_HOME}/MATH/lib


function lammps_install()
{
    cd ${WORK_HOME}/lammps-5Jun19/src
    sed -ri "s|-g -O3|-g -O3 -march=armv8.2-a -mcpu=tsv110 -L${MATH_LIB} -lmathlib|g" MAKE/OPTIONS/Makefile.g++_openmpi
    sed -ri "s|LIB = |LIB = -march=armv8.2-a -mcpu=tsv110 -L${MATH_LIB} -lmathlib|g" MAKE/OPTIONS/Makefile.g++_openmpi
    sed -ri "s/FFT_INC =/#FFT_INC =/g" MAKE/OPTIONS/Makefile.g++_openmpi
    sed -ri "s/FFT_PATH =/#FFT_PATH =/g" MAKE/OPTIONS/Makefile.g++_openmpi
    sed -ri "s/FFT_LIB =/#FFT_LIB =/g" MAKE/OPTIONS/Makefile.g++_openmpi
    cat >> ${WORK_HOME}/lammps-5Jun19/src/MAKE/OPTIONS/Makefile.g++_openmpi << EOF
FFT_INC = -DFFT_FFTW -I${FFTW_DIR}/include
FFT_PATH =   -L${FFTW_DIR}/lib
FFT_LIB =    -L${FFTW_DIR}/lib -lfftw3
EOF
    make yes-std
    make no-lib
    { make -j ${max_jobs} g++_openmpi ; } 2>&1 | tee -a make.log
    mkdir -p ${INS_HOME_PATH}/LAMMPS/bin
    cp lmp_g++_openmpi ${INS_HOME_PATH}/LAMMPS/bin
    mv make.log ${INS_HOME_PATH}/LAMMPS
}

lammps_install
