#!/bin/bash
set -e
HOST=$PWD
INS_HOME_PATH="${HOST}/application"
SRC_HOME_PATH="${HOST}/source"


#设置环境
echo "export NETCDF_DIR=${HOST}/NETCDF" >> nemo-env.sh
echo "export HDF_DIR=${HOST}/HDF5" >> nemo-env.sh
echo "export XIOS_DIR=${HOST}/XIOS_" >> nemo-env.sh
echo "export MATH_LIB=${HOST}/MATH/lib" >> nemo-env.sh
source ./nemo-env.sh

max_jobs=$(nproc)
if [[ ${max_jobs} -gt 32 ]]; then
    max_jobs=32
fi

function nemo_install()
{
    cd ${SRC_HOME_PATH}/NEMO/releases/release-3.6/NEMOGCM
    cat > ${SRC_HOME_PATH}/NEMO/releases/release-3.6/NEMOGCM/ARCH/arch-aarch64_gnu.fcm << EOF
%NCDF_HOME           $NETCDF_DIR
%HDF5_HOME           $HDF_DIR
%XIOS_HOME           $XIOS_DIR

%NCDF_INC            -I%NCDF_HOME/include -I%HDF5_HOME/include
%NCDF_LIB            -L%HDF5_HOME/lib -L/%NCDF_HOME/lib -lnetcdf -lnetcdff
%XIOS_INC            -I%XIOS_HOME/include
%XIOS_LIB            -L%XIOS_HOME/lib -lxios

%CPP                 cpp -Dkey_nosignedzero
%FC                  mpif90 -c -cpp
%FCFLAGS             -mcpu=tsv110 -fdefault-real-8 -fdefault-double-8 -O3 -funroll-all-loops -fcray-pointer -ffree-line-length-none -g
%FFLAGS              %FCFLAGS
%LD                  mpif90
%LDFLAGS             -lstdc++
%FPPFLAGS            -P -C -traditional
%AR                  ar
%ARFLAGS             rs
%MK                  gmake
%USER_INC            %XIOS_INC %NCDF_INC
%USER_LIB            %XIOS_LIB %NCDF_LIB -mcpu=tsv110 -L${MATH_LIB} -lmathlib

%CC                  cc
%CFLAGS              -O0
EOF

    chmod 700 ./ -R
    cd ./CONFIG/
    { ./makenemo -m aarch64_gnu -j ${max_jobs} -r AMM12 -n 'MY_AMM12' add_key "key_nosignedzero" ; } 2>&1 |tee install-1.log
    mkdir -p ${INS_HOME_PATH}/NEMO/bin
    cp ${SRC_HOME_PATH}/NEMO/releases/release-3.6/NEMOGCM/CONFIG/MY_AMM12/EXP00/opa ${INS_HOME_PATH}/NEMO/bin
    mv install-*.log ${INS_HOME_PATH}/NEMO
}

nemo_install
