#!/bin/bash
set -e
HOST=$PWD
INS_HOME_PATH="${HOST}/application"
SRC_HOME_PATH="${HOST}/source"


#设置环境
echo "export NETCDF_DIR=${HOST}/NETCDF" >> nemo-env.sh
echo "export HDF_DIR=${HOST}/HDF5" >> nemo-env.sh
echo "export XIOS_DIR=${HOST}/XIOS_" >> nemo-env.sh
source ./nemo-env.sh

max_jobs=$(nproc)
if [[ ${max_jobs} -gt 32 ]]; then
    max_jobs=32
fi

function nemo_install()
{
    set +e
    # wget -o 指定的包会解压异常（CRC）
    unzip nemo-3.6.zip -d source/ -x /
    set -e
    cd ${SRC_HOME_PATH}/NEMO/releases/release-3.6/NEMOGCM
    cat > ./ARCH/arch-aarch64_gnu.fcm << EOF
#generic gfortran compiler options for linux
#NCDF_INC    netcdf include file
#NCDF_LIB    netcdf library
#FC          Fortran compiler command
#FCFLAGS     Fortran compiler flags
#FFLAGS      Fortran 77 compiler flags
#LD          linker
#LDFLAGS     linker flags, e.g. -L<lib dir> if you have libraries in a
#FPPFLAGS    pre-processing flags
#AR          assembler
#ARFLAGS     assembler flags
#MK          make
#USER_INC    additional include files for the compiler,  e.g. -I<include dir>
#USER_LIB    additional libraries to pass to the linker, e.g. -l<library>
#CC          C compiler used to compile conv for AGRIF
#CFLAGS      compiler flags used with CC
#
#Note that:
# - unix variables "$..." are accpeted and will be evaluated before calling fcm.
# - fcm variables are starting with a % (and not a $)
%NCDF_HOME           $NETCDF_DIR
%HDF5_HOME           $HDF_DIR
%XIOS_HOME           $XIOS_DIR
%NCDF_INC            -I%NCDF_HOME/include -I%HDF5_HOME/include
%NCDF_LIB            -L%HDF5_HOME/lib -L/%NCDF_HOME/lib -lnetcdf -lnetcdff
%XIOS_INC            -I%XIOS_HOME/inc
%XIOS_LIB            -L%XIOS_HOME/lib -lxios
%CPP                 cpp
%FC                  mpifort
%FCFLAGS             -cpp -r8 -O3 -mcpu=native
%FFLAGS              %FCFLAGS
%LD                  mpif90
%LDFLAGS             -lstdc++
%FPPFLAGS            -P -C -traditional
%AR                  ar
%ARFLAGS             rs
%MK                  gmake
%USER_INC            %XIOS_INC %NCDF_INC
%USER_LIB            %XIOS_LIB %NCDF_LIB
%CC                  clang
%CFLAGS              -O0

EOF
    chmod 700 ./ -R
    cd ./CONFIG/
    { ./makenemo -m aarch64_gnu -j ${max_jobs} -r AMM12 -n 'MY_AMM12' add_key "key_nosignedzero" ; } 2>&1 |tee install-1.log
    mkdir -p ${INS_HOME_PATH}/NEMO/bin
    cp ${HOST}/namelist_cfg.patch ./MY_AMM12/EXP00/
    cd ./MY_AMM12/EXP00/
    patch -p0 namelist_cfg namelist_cfg.patch
    cp ${SRC_HOME_PATH}/NEMO/releases/release-3.6/NEMOGCM/CONFIG/MY_AMM12/EXP00/namelist_cfg ${INS_HOME_PATH}/NEMO/bin
}

nemo_install
