#!/bin/bash

### Description: This is a Frame Script, which is used for HPC Application automatic compilation and installation !!! ###
###                                         Version Number: V 1.0                                                     ###
###                                       Released Date: 2021/4/23                                                   ###
#########################################################################################################################

######################################## Don't Modify####################################################################

### Global Definition ###
# 1. Modify "APPLICATION" and "VERSION"
APPLICATION="nwchem"
VERSION="6.8.1"
GA_VERSION="5.6.5"
HOST=$PWD
INS_HOME_PATH="${HOST}/application"
SRC_HOME_PATH="${HOST}/source"
ENV_PATH="${HOST}/env"
Architecture=`uname -m`

### Define the src path and installation path ###
#=== Build Path ===
mkdir -p ${SRC_HOME_PATH}/${APPLICATION}/${VERSION}/
mkdir -p ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}
mkdir -p ${ENV_PATH}/${Architecture}/${APPLICATION}


#定义编译和安装相关函数
function NWChem_decompress()
{
   #判断源码包是否下载
   if [ ! -f ${HOST}/${APPLICATION}-${VERSION}-release.tar.gz ]; then
        echo "Package download failed, please check the network connection and others"
        exit 1
   else
        echo "The NWChem Source Packge is allready exiting, no need to download again"
   fi
   #解压源码包
   if [ ! -d ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/nwchem-${VERSION}-release ]; then
      tar -zxvf ${HOST}/${APPLICATION}-${VERSION}-release.tar.gz \
      -C ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}
      sed -ri '23 s/64/1024/g' ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/nwchem-${VERSION}-release/src/util/GNUmakefile
      sed -i '24i \   LIB_DEFINES += -DNWCHEM_LONG_PATHS' ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/nwchem-${VERSION}-release/src/util/GNUmakefile
   fi
}


function  GlobalArray_decompress()
{
   #判断源码包是否下载
   if [ ! -f ${HOST}/ga-${GA_VERSION}.tar.gz ]; then
        echo "Package download failed, please check the network connection and others"
        exit 1
   else
        echo "The GlobalArray Source Packge is allready exiting, no need to download again"
   fi
   #解压源码包
   if [ ! -d ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/nwchem-${VERSION}-release/src/tools/ga-${GA_VERSION} ]; then
      cp ga-${GA_VERSION}.tar.gz ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/nwchem-${VERSION}-release/src/tools
      tar zxvf ${HOST}/ga-${GA_VERSION}.tar.gz \
      -C ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/nwchem-${VERSION}-release/src/tools
   fi    
}

function nwchem_make()
{
     cd ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/nwchem-${VERSION}-release/src
     make nwchem_config
     make FC=gfortran >& make.log
}


### Begin to download all packages ###
# 3. Change to your own define function name
NWChem_decompress
GlobalArray_decompress


### Hardware Architecture, Select Baseline Combination, Load Module File ###
case $Architecture in
aarch64)
echo ""
echo "The hardware platform is aarch64"
echo ""

if [ x[select_bc_value]==x ];then
        select_bc_value="1"
fi

for select_bc in $select_bc_value
do
        case $select_bc in
        1) #(1) gcc9.3.0 + openmpi4.0.3
        export COMPILER=gcc9.3.0
        export MPI=omp4.0.3
        export LIB=os_lib

        export CC=`which gcc`
        export CXX=`which g++`
        export FC=`which gfortran`

        export NWCHEM_TOP=${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/nwchem-${VERSION}-release
        export NWCHEM_TARGET=LINUX64
        export NWCHEM_MODULES=all
        export USE_MPI=y
        export USE_INTERNALBLAS=y

        #编译和安装
        nwchem_make
        
        ;;

esac
        #检查是否安装成功
        ls ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/nwchem-${VERSION}-release/bin/LINUX64/nwchem
        if [ $? -ne 0 ];then
        echo "Can not generate the nwchem,please check your configuration !"
                exit 1
        fi

done

;;
*)
echo "Please check you Architecture!"
exit 1
;;
esac
