#!/bin/bash

### Description: This is a Frame Script, which is used for HPC Application automatic compilation and installation !!! ###
###                                         Version Number: V 1.0                                                     ###
###                                       Released Date: 2019/11/20                                                   ###
#########################################################################################################################

######################################## Don't Modify####################################################################

### Global Definition ###
# 1. Modify "APPLICATION" and "VERSION" 
APPLICATION="OpenFOAM"
VERSION="v1906"
HOST=$PWD
INS_HOME_PATH="${HOST}/application"
SRC_HOME_PATH="${HOST}/source"
ENV_PATH="${HOST}/env"
Architecture=`uname -m`

### Define the src path and installation path ###
#=== Build Path ===
mkdir -p ${SRC_HOME_PATH}/${APPLICATION}/${VERSION}/
mkdir -p ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}
mkdir -p ${ENV_PATH}/${Architecture}/${APPLICATION}

function openfoam_unpack()
{
   if [ ! -d ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/${APPLICATION}-${VERSION} ]; then
      tar -zxvf ${APPLICATION}-${VERSION}.tgz \
      -C ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}
   fi
}

function thirdparty_unpack()
{
   if [ ! -d ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/ThirdParty-${VERSION} ]; then
      tar -zxvf ThirdParty-${VERSION}.tgz \
      -C ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}
   fi
}


### Begin to unpack all packages ###
# 3. Change to your own define function name
openfoam_unpack
thirdparty_unpack
mpi=ompi
mpi_version=4.0.3

function ompi_install()
{
        ompi_install_path=${INS_HOME_PATH}/${Architecture}/mpi/${mpi}${mpi_version}
        [ -d ${ompi_install_path} ] && break
        mkdir -p /tmp/openmpi-${mpi_version}
        tar -xvf openmpi-${mpi_version}.tar.gz -C /tmp/openmpi-${mpi_version} --strip-components 1
        cd /tmp/openmpi-${mpi_version}
        [ -f config.log ] && make distclean

        ./configure --prefix=${ompi_install_path} --enable-pretty-print-stacktrace --enable-mpi1-compatibility CC=gcc CXX=g++ FC=gfortran
        make -j 16
        make install

}

### Modify OpenFOAM WM_OPTION to show MPI ####
# 4. Modify source code file
sed -i "s/WM_COMPILER\$WM_PRECISION_OPTION/WM_COMPILER\$WM_MPLIB\$WM_PRECISION_OPTION/g" ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/${APPLICATION}-${VERSION}/etc/config.csh/settings
sed -i "s/WM_COMPILER\$WM_PRECISION_OPTION/WM_COMPILER\$WM_MPLIB\$WM_PRECISION_OPTION/g" ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/${APPLICATION}-${VERSION}/etc/config.sh/settings
################################################# Modification DONE##################
 
### Dependency Package Installation ###
# 5. Modify yum command
ompi_install

### Hardware Architecture, Select Baseline Combination, Load Module File ###
case $Architecture in
aarch64)
echo ""
echo "The hardware platform is aarch64"
echo ""

if [ x[select_bc_value]==x ];then
        select_bc_value="1"
fi

for select_bc in $select_bc_value
do
        case $select_bc in
	1)	#(1) GNU + HPCX + OS_LIB#
	export PATH=${ompi_install_path}/bin:$PATH
	export LD_LIBRARY_PATH=${ompi_install_path}/lib:$LD_LIBRARY_PATH
	export COMPILER=gnu-4.8.5
	export MPI=ompi-4.0.3
	export LIB=os_lib	
	
        # Create the main application install path #
        echo "export PATH=${ompi_install_path}/bin:$PATH" > ${ENV_PATH}/${Architecture}/${APPLICATION}/env_${COMPILER}-${MPI}-${LIB}
        echo "export LD_LIBRARY_PATH=${ompi_install_path}/lib:$LD_LIBRARY_PATH" >> ${ENV_PATH}/${Architecture}/${APPLICATION}/env_${COMPILER}-${MPI}-${LIB}

        # Begin to compile openfoam #
        cd ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/${APPLICATION}-${VERSION}
        if [ ! -f "etc/bashrc_${COMPILER}-${MPI}-${LIB}" ]; then
                cp etc/bashrc etc/bashrc_${COMPILER}-${MPI}-${LIB}
                echo "source $PWD/etc/bashrc_${COMPILER}-${MPI}-${LIB}" >> ${ENV_PATH}/${Architecture}/${APPLICATION}/env_${COMPILER}-${MPI}-${LIB}
                source etc/bashrc_${COMPILER}-${MPI}-${LIB}
        else
                echo "OpenFOAM_${COMPILER}-${MPI}-${LIB} has been installed"
                continue
        fi

	;;
	*)
	echo "Please input the correct select_bc_value!"
        exit 1
	;;
	esac

        # start compiling

        cd ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/${APPLICATION}-${VERSION}

        if [ $(lscpu | grep "^CPU(s):" | awk '{print $2}') -lt 16 ];then
                ./Allwmake -j $(lscpu | grep "^CPU(s):" | awk '{print $2}') 2>&1 | tee -a compile_${COMPILER}-${MPI}-${LIB}.log
        else
                ./Allwmake -j 40 2>&1 | tee -a compile_${COMPILER}-${MPI}-${LIB}.log
        fi

        # check the openfoam
        foamInstallationTest
        if [ $? -ne 0 ];then
        echo "Can not generate the openfoam,please check your configuration !"
                exit 1
        fi

done


;;
*)
echo "Please check you Architecture!"
exit 1
;;
esac
