#!/bin/bash

HOST=$PWD
INS_HOME_PATH="${HOST}/application"

export CC=`which mpicc`
export CXX=`which mpic++`
export FC=`which mpifort`

HDF5=${HOST}/HDF5
NETCDF=${HOST}/NETCDF
PNETCDF=${HOST}/PNETCDF
MATH=${HOST}/MATH

function hdf5_install()
{
   cd ${HOST}/hdf5-1.10.1
   ./configure --prefix=${HDF5} --build=aarch64-unknown-linux-gnu --enable-fortran --enable-static=yes --enable-parallel --enable-shared CC=mpicc CXX=mpicxx FC=mpifort F77=mpifort
   make -j 16 && make install
}

function pnetcdf_install()
{
   cd ${HOST}/parallel-netcdf-1.9.0
   ./configure --prefix=${PNETCDF} --build=aarch64-unknown-linux-gnu  CFLAGS="-fPIC -DPIC" CXXFLAGS="-fPIC -DPIC" FCFLAGS="-fPIC" FFLAGS="-fPIC" CC=mpicc CXX=mpicxx FC=mpifort F77=mpifort
   make -j 16 && make install
}

function netcdf_install()
{
   cd ${HOST}/netcdf-c-4.4.1.1
   ./configure --prefix=${NETCDF} --build=aarch64-unknown-linux-gnu --enable-shared --enable-netcdf-4 --enable-dap --with-pic --disable-doxygen --enable-static --enable-pnetcdf --enable-largefile CC=mpicc CXX=mpicxx FC=mpifort F77=mpifort CPPFLAGS="-I$HDF5/include -I$PNETCDF/include" LDFLAGS="-L$HDF5/lib -L$PNETCDF/lib -Wl,-rpath=$HDF5/lib -Wl,-rpath=$PNETCDF/lib" CFLAGS="-L$HDF5/lib -L$PNETCDF/lib -I$HDF5/include -I$PNETCDF/include"
   make -j 16 && make install
}

function netcdff_install()
{
   cd ${HOST}/netcdf-fortran-4.4.1
   ./configure --prefix=$NETCDF --build=aarch64-unknown-linux-gnu --enable-shared --with-pic --disable-doxygen --enable-largefile --enable-static CC=mpicc CXX=mpicxx FC=mpifort F77=mpifort CPPFLAGS="-I$HDF5/include -I$NETCDF/include" LDFLAGS="-L$HDF5/lib -L$NETCDF/lib -Wl,-rpath=$HDF5/lib -Wl,-rpath=$NETCDF/lib" CFLAGS="-L$HDF5/lib -L$NETCDF/lib -I$HDF5/include -I$NETCDF/include" CXXFLAGS="-L$HDF5/lib -L$NETCDF/lib -I$HDF5/include -I$NETCDF/include" FCFLAGS="-L$HDF5/lib -L$NETCDF/lib -I$HDF5/include -I$NETCDF/include"
   make -j 16 && make install
}

function MATH_install()
{
   cd ${HOST}/optimized-routines-201910
   cp config.mk.dist config.mk
   make -j 16
   cp -r ./build/* $MATH
}


hdf5_install
pnetcdf_install
netcdf_install
netcdff_install
MATH_install
