#!/bin/bash

HOST=$PWD
INS_HOME_PATH="${HOST}/application"

export CC=clang
export CXX=clang++
export FC=flang
export MPICC=mpicc
export MPICXX=mpicxx
export MPIFC=mpifort

HDF5=${HOST}/HDF5
NETCDF=${HOST}/NETCDF
PNETCDF=${HOST}/PNETCDF
ZLIB=${HOST}/ZLIB


function hdf5_install()
{
   cd ${HOST}/hdf5-1.12.1
   CC=mpicc CXX=mpicxx FC=mpifort F77=mpi77 CFLAGS="-fPIC -DPIC" FCFLAGS="-fPIC -DPIC" FFLAGS="-fPIC" \
   ./configure --prefix=${HDF5} --with-zlib=${ZLIB} --enable-shared --enable-fortran --enable-static --enable-parallel
   sed -i 's@\\\$wl-soname@-install_name@g' libtool
   make -j $(nproc) && make -j $(nproc) install
}

function pnetcdf_install()
{
   cd ${HOST}/parallel-netcdf-1.9.0
  CC=mpicc CXX=mpicxx FC=mpifort F77=mpifort CFLAGS="-fPIC -DPIC" FCFLAGS="-fPIC" FFLAGS="-fPIC" CXXFLAGS="-fPIC -DPIC" SEQ_CC=mpicc \
  ./configure --prefix=${PNETCDF} --enable-shared
  make -j$(nproc) && make -j$(nproc) install
}

function netcdf_c_install()
{
    cd ${HOST}/netcdf-c-4.4.1.1
    HDF5_LIB=${HDF5}/lib ZLIB_LIB=${ZLIB}/lib PNETCDF_LIB=${PNETCDF}/lib \
    HDF5_INCLUDE=${HDF5}/include ZLIB_INCLUDE=${ZLIB}/include PNETCDF_INCLUDE=${PNETCDF}/include \
    CC=mpicc CXX=mpicxx CPPFLAGS="-I${HDF5_INCLUDE} -I${ZLIB_INCLUDE} -I${PNETCDF_INCLUDE}" \
    CFLAGS="-I${HDF5_INCLUDE}  -I${ZLIB_INCLUDE} -I${PNETCDF_INCLUDE} -fPIC" \
    LDFLAGS="-L${HDF5_LIB}  -L${ZLIB_LIB} -L${PNETCDF_LIB}" \
    ./configure --prefix=${NETCDF} --enable-shared --enable-netcdf-4 --enable-dap --with-pic --disable-doxygen --enable-pnetcdf --enable-static --enable-largefile
    make -j$(nproc) && make -j$(nproc) install
}

function netcdf_fortran_install()
{
    cd ${HOST}/netcdf-fortran-4.4.1
    ./configure --prefix=$NETCDF CPPFLAGS="-I$HDF5/include -I$NETCDF/include" LDFLAGS="-L$HDF5/lib -L$NETCDF/lib" \
    --build=aarch64-unknown-linux-gnu --enable-static=yes \
    --enable-shared CFLAGS="-O3 -fPIC -Wno-incompatible-pointer-types-discards-qualifiers -Wno-non-literal-conversion" \
    FCFLAGS="-O3 -fPIC" LDFLAGS="-Wl,--build-id" CC=mpicc FC=mpif90 CXX=mpicxx
    sed -i '10118d' libtool
    sed -i '10118i wl="-Wl,"' libtool
    sed -i '10270d' libtool
    sed -i '10270i wl="-Wl,"', libtool
    make -j$(nproc) && make -j$(nproc) install
}

function zlib_install()
{
    cd ${HOST}/zlib-1.2.11
    CC=clang ./configure --prefix=${ZLIB}
    make -j $(nproc) && make -j $(nproc) install
}

zlib_install
hdf5_install
pnetcdf_install
netcdf_c_install
netcdf_fortran_install

